//////////////////////////////////////////////////////////////////////
//  CVTExtRow  
//
// :   aka ADirks
// : e-mail: adirks@ngs.ru, ICQ: 13472890
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VTExtended.h"


HANDLE CVTExtHeap::hHeap = NULL;
static CVTExtHeap Heap;

CVTExtHeap::CVTExtHeap()
{
	if( !hHeap )
	{
		hHeap = HeapCreate(HEAP_NO_SERIALIZE, 0, 0);
	}
}

CVTExtHeap::~CVTExtHeap()
{
	if( hHeap )
		HeapDestroy(hHeap);
}


#if CVTExtRow_VARIANT == 1

CVTExtRow::CVTExtRow(int numValues)
{
	SetSize(numValues);
	while( --numValues >= 0 ) SetAt(numValues, new CValue);
}

CVTExtRow::~CVTExtRow()
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		delete GetAt(i);
	}
}

int CVTExtRow::NewColumn()
{
	return Add(new CValue);
}

void CVTExtRow::RemoveColumn(int nCol)
{
	delete GetAt(nCol);
	RemoveAt(nCol);
}

CVTExtRow& CVTExtRow::operator= (CVTExtRow const& SrcRow)
{
	int i = min(GetUpperBound(), SrcRow.GetUpperBound());
	while( i >= 0 )
	{
		SetValue(i, ((CVTExtRow&)SrcRow).GetValue(i));
		i--;
	}
	return *this;
}

//====================================================================================
#elif CVTExtRow_VARIANT == 2
CValue* CVTExtRow::AllocMem(int nSize)
{
	CValue* pValues = (CValue*)malloc(sizeof(CValue) * nSize);
	CValue* ptr = pValues;
	while( nSize > 0 )
	{
		new(ptr) CValue; //  
		ptr++;
		nSize--;
	}
	return pValues;
}

void CVTExtRow::FreeMem(CValue* pValues, int nSize)
{
	if( pValues )
	{
		CValue* ptr = pValues;
		for( int i = 0; i < nSize; i++ )
		{
			ptr->~CValue();
			ptr++;
		}
		free(pValues);
	}
}

CVTExtRow::CVTExtRow(int numValues)
{
	nSize = numValues;
	if( nSize > 0 )
	{
		pValues = AllocMem(nSize);
	}
	else
	{
		nSize = 0;
		pValues = NULL;
	}
}

CVTExtRow::~CVTExtRow()
{
	FreeMem(pValues, nSize);
}


/*
int CVTExtRow::NewColumn()
{
	int nNewCol = nSize;
	nSize++;
	
	CValue* pNewValues = malloc(sizeof(CValue) * nSize);
	memcpy(pNewValues, pValues, sizeof(CValue) * (nSize-1));
	free(pValues);

	new(pNewValues+nNewCol) CValue;
	
	pValues = pNewValues;

	return nNewCol;
}
*/

int CVTExtRow::NewColumn()
{
	int nNewCol = nSize;
	nSize++;
	CValue* pNewValues = AllocMem(nSize);
	//new(pNewValues+nNewCol) CValue;

	for( int i = 0; i < nNewCol; i++ )
		pNewValues[i] = pValues[i];
	
	FreeMem(pValues, nSize-1);

	pValues = pNewValues;

	return nNewCol;
}

void CVTExtRow::RemoveColumn(int nCol) 
{
	(pValues + nCol)->~CValue();
	memcpy(pValues+nCol, pValues+nCol+1, sizeof(CValue) * (nSize - nCol - 1));
	nSize--;
}



CVTExtRow& CVTExtRow::operator= (CVTExtRow const& SrcRow)
{
	int nMinSz = min(nSize, SrcRow.nSize);
	for( int i = 0; i < nMinSz; i++ )
	{
		SetValue(i, ((CVTExtRow&)SrcRow).GetValue(i));
	}
	return *this;
}


//====================================================================================
#elif CVTExtRow_VARIANT == 3

CValue* CVTExtRow::AllocMem(int nNewSize)
{
	//  ,   
	//      
	#define EXESSIVE_ROWSIZE_HINT 1

	CValue *pV, *ptr;
	if( nNewSize )
	{
		HANDLE hHeap = CVTExtHeap::Get();
		pV = (CValue*) HeapAlloc(hHeap, HEAP_NO_SERIALIZE, sizeof(CValue) * (nNewSize + EXESSIVE_ROWSIZE_HINT));

		if ( pV )
		{
			ptr = pV;
			for ( int i =0; i < nNewSize; i++ )
				new( ptr++ ) CValue; //  
		}
	}
	else
		pV = NULL;

	return pV;
}

void CVTExtRow::FreeMem(CValue* pV, int nCurrentSize)
{
	if( pV )
	{
		CValue *ptr = pV;
		for( int i = 0; i < nCurrentSize; i++ )
			(ptr++)->~CValue();

		HANDLE hHeap = CVTExtHeap::Get();
		HeapFree(hHeap, HEAP_NO_SERIALIZE, (LPVOID)pV);
	}
}

//      
CValue* CVTExtRow::ReAllocMem(CValue* pCurrentV, int nCurrentSize, int nNewSize)
{
	CValue *pV, *ptr;
	if( pCurrentV && nNewSize )
	{
		HANDLE hHeap = CVTExtHeap::Get();

		if( HeapSize(hHeap, HEAP_NO_SERIALIZE, pCurrentV) >= nNewSize * sizeof(CValue) )
			//   ;       
			pV = pCurrentV;
		else
			//   HEAP_REALLOC_IN_PLACE_ONLY -   CNumeric
			pV = (CValue*) HeapReAlloc(hHeap, HEAP_REALLOC_IN_PLACE_ONLY | HEAP_NO_SERIALIZE, (LPVOID)pCurrentV, sizeof(CValue) * nNewSize);

		if( pV )
		{
			ptr = pV + nCurrentSize;
			for ( int i = nCurrentSize; i < nNewSize; i++ )
				new( ptr++ ) CValue; //  
		}
	}
	else
		pV = NULL;
	
	return pV;
}

CVTExtRow::CVTExtRow(int numValues)
{
	if( numValues > 0 )
	{
		nSize = numValues;
		pValues = AllocMem(nSize);
	}
	else
	{
		nSize = 0;
		pValues = NULL;
	}
}

CVTExtRow::~CVTExtRow()
{
	FreeMem(pValues, nSize);

	nSize = 0;
	pValues = NULL;
}

int CVTExtRow::NewColumn()
{
	int nNewCol = nSize++;
	CValue* pNewValues;

	pNewValues = ReAllocMem(pValues, nNewCol, nSize);

	if( !pNewValues )
	{
		//     -   
		pNewValues = AllocMem(nSize);

		for( int i = 0; i < nNewCol; i++ )
			pNewValues[i] = pValues[i];
		
		FreeMem(pValues, nSize - 1);
	}
	pValues = pNewValues;

	return nNewCol;
}

void CVTExtRow::RemoveColumn(int nCol) 
{
	CValue *ptr = pValues + nCol;
	ptr->~CValue();
	
	nSize--;
	for( int i = nCol; i < nSize; i++, ptr++ )
	{
		*(ptr) = *(ptr + 1);
	}
}


CVTExtRow& CVTExtRow::operator= (CVTExtRow& SrcRow)
{
	int nMinSz = min(nSize, SrcRow.nSize);
	for( int i = 0; i < nMinSz; i++ )
	{
		SetValue(i, SrcRow.GetValue(i));
	}
	return *this;
}

#endif
