//////////////////////////////////////////////////////////////////////
//         CVTExtended
//
// :   aka ADirks
// : e-mail: adirks@ngs.ru, ICQ: 13472890
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VTExtended.h"
#include "../StringArrayEx.h"

/////////////////////////////////////////////////////////////
////  CVTIndexRecord  ///////////////////////////////////////
/////////////////////////////////////////////////////////////
int numeric_compare(int a, int b)
{
	if( a < b ) return -1;
	else if( a > b ) return 1;
	return 0;
}

int string_compare(CString const& left, CString const& right, TVTIndexDescrRecord const& IdxRec)
{
	if( IdxRec.TrimStrings )
	{
		CString lstr = (LPCSTR)left;
		lstr.TrimRight();
		lstr.TrimLeft();
		CString rstr = (LPCSTR)right;
		rstr.TrimRight();
		rstr.TrimLeft();
		if( IdxRec.NoCaseStringCompare )
			return lstr.CollateNoCase(rstr);
		else
			return strcmp(lstr, rstr);
	}
	else if( IdxRec.NoCaseStringCompare )
		return left.CollateNoCase(right);

	return strcmp(left, right);
}

int date_compare(CDate const& d1, CDate const& d2)
{
	int comp_res = numeric_compare(d1.GetYear(), d2.GetYear());
	if( comp_res ) return comp_res;
	comp_res = numeric_compare(d1.GetMonth(), d2.GetMonth());
	if( comp_res ) return comp_res;
	comp_res = numeric_compare(d1.GetMonthDay(), d2.GetMonthDay());
	return comp_res;
}

int time_compare(CEventTime const& t1, CEventTime const& t2)
{
	if( t1 < t2 ) return -1;
	else if( t1 > t2 ) return 1;
	return 0;
}

int inner_compare(CValue &left, CValue &right)
{
	//*
	int diff;

	if( !(diff = left.GetTypeID() - right.GetTypeID()) )
		if( !(diff = left.m_ObjID.GetlObjID() - right.m_ObjID.GetlObjID()) )
			return strcmp(left.m_ObjID.DBSign.Sign, right.m_ObjID.DBSign.Sign);
	if( diff < 0 ) return -1;
	if( diff ) return 1;
	return 0;
	//*/
	/*
	CString l_str, r_str;
	left.SaveToString(l_str);
	right.SaveToString(r_str);
	return strcmp(l_str, r_str);
	//*/
}

int CValue_compare(CValue &left, CValue &right, TVTIndexDescrRecord const& IdxRec)
{
	if( left.GetTypeCode() == right.GetTypeCode() )
	{
		switch( left.GetTypeCode() )
		{
		case NUMBER_TYPE_1C:
			return left.GetNumeric().Compare(right.GetNumeric());
			break;
		case STRING_TYPE_1C:
			return string_compare(left.GetString(), right.GetString(), IdxRec);
			break;
		case DATE_TYPE_1C:
			{
			CDate &d1 = left.GetDate(), &d2 = right.GetDate();
			return date_compare(d1, d2);
			}
			break;
		case REFERENCE_TYPE_1C:
			{
			if( IdxRec.CompareType == CmpByInnerRepr )
				return inner_compare(left, right);
			int comp_res = string_compare(left.Format(), right.Format(), IdxRec);
			if( comp_res == 0 )
				comp_res = inner_compare(left, right);
			return comp_res;
			}
			break;
		case DOCUMENT_TYPE_1C:
			if( IdxRec.CompareType == CmpByInnerRepr )
			{
				return inner_compare(left, right);
			}
			else
			{
				CDate d1(0,0,0), d2(0,0,0);
				CEventTime t1, t2;
				GetDateTimeFromValue(left, d1, t1);
				GetDateTimeFromValue(right, d2, t2);
				int comp_res = date_compare(d1, d2);
				if( comp_res == 0 ) comp_res = time_compare(t1, t2);
				return comp_res;
			}
			break;
		default:
			if( IdxRec.CompareType == CmpByInnerRepr )
				return inner_compare(left, right);
			else
				return string_compare(left.Format(), right.Format(), IdxRec);
		}
	}
	else
	{
		return numeric_compare(left.GetTypeCode(), right.GetTypeCode());
	}
}

int CVTIndexRecord::compare(CVTIndexRecord& RightRecord)
{
	int sz = IndexFields->size();
	int comp_res;
	for( int i = 0; i < sz; i++ )
	{
		TVTIndexDescrRecord &IdxRec = IndexFields->operator[](i);

		CValue &left = Row->GetValue(IdxRec.ColumnNumber);
		CValue &right = RightRecord.Row->GetValue(IdxRec.ColumnNumber);
		try{
			comp_res = CValue_compare(left, right, IdxRec);
		} catch(...) 
		{
			RuntimeError("      . rows: %p vs %p; values: %p vs %p", 
				Row, RightRecord.Row, &left, &right);
		}

		if( comp_res != 0 ) return comp_res * IdxRec.Direction;
	}

	return 0;
}


/////////////////////////////////////////////////////////////
////  CVTExtPosition  ///////////////////////////////////////
/////////////////////////////////////////////////////////////
CVTExtPosition& CVTExtPosition::operator= (CVTExtIndex& Index)
{
	CurrentRow = Index.CurrentRow;
	pCurrentRow = Index.pCurrentRow;
	CurrentNode = Index.CurrentNode;
	IndexInNode = Index.IndexInNode;
	SelStarted = Index.SelStarted;
	return *this;
}

/////////////////////////////////////////////////////////////
////  CVTExtIndexFields  ////////////////////////////////////
/////////////////////////////////////////////////////////////
CVTExtIndexFields::CVTExtIndexFields(CString& IndexExpr, CVTExtended* pVT)
{
	Build(IndexExpr, pVT);
}

bool CVTExtIndexFields::Build(CString IndexExpr, CVTExtended* pVT)
{
	clear();

	CStringArrayEx Fields;
	Fields.FillSeparateString(IndexExpr, ",");
	int nFields = Fields.GetSize();

	if( nFields == 0 )
	{
		return true; //  "" , ..     
	}

	resize(nFields);

	for( int i = 0; i < nFields; i++ )
	{
		CString& ColName = Fields[i];
		int direction = 1;
		TCompareType CompareType = CmpByStringRepr;
		bool TrimStrings = false, NoCaseStringCompare = false;
		
		operator[](i).CompareType = ColName.Find('*') < 0 ? CmpByStringRepr : CmpByInnerRepr;
		operator[](i).Direction = ColName.Find('-') < 0 ? 1 : -1;
		operator[](i).TrimStrings = ColName.Find('#') >= 0;
		operator[](i).NoCaseStringCompare = ColName.Find('^') >= 0;
		operator[](i).SumsBySubcontoGroups = ColName.Find('&') >= 0;
		
		ColName.Replace("-", "");
		ColName.Replace("*", "");
		ColName.Replace("#", "");
		ColName.Replace("^", "");
		ColName.Replace("&", "");

		if( pVT )
		{
			int col_num = pVT->ColumnNumber(ColName, false);
			if( col_num < 0 )
				RuntimeError(" '%s'  !", ColName);

			operator[](i).ColumnNumber = col_num;
		}
	}

	return true;
}

/////////////////////////////////////////////////////////////
////  CVTExtIndex  //////////////////////////////////////////
/////////////////////////////////////////////////////////////
CVTExtIndex::CVTExtIndex(LPCSTR Name, CString const& IndexExpression, bool bUnique, CVTExtended* vt)
:KeyRow(vt->GetColumnsCount())
{
	SetName(Name);
	pVT = vt;
	pFilter = NULL;

	IndexFields.Build(IndexExpression, pVT);
	NoRepeat = bUnique;
	Reindex();
	SelStarted = false;
}

CVTExtIndex::CVTExtIndex(LPCSTR Name, CVTExtIndexFields const& TheIndexFields, bool bUnique, CVTExtended* vt)
:KeyRow(vt->GetColumnsCount())
{
	SetName(Name);
	pVT = vt;
	pFilter = NULL;

	IndexFields = TheIndexFields;
	NoRepeat = bUnique;
	Reindex();
	SelStarted = false;
}

CVTExtIndex::CVTExtIndex(LPCSTR Name, CVTExtended* vt)
:KeyRow(vt->GetColumnsCount())
{
	SetName(Name);
	pVT = vt;
	pFilter = NULL;
	ResetPosition();
	SelStarted = false;
}

CVTExtIndex::~CVTExtIndex()
{
	if( pFilter != NULL ) delete pFilter;
}

void CVTExtIndex::ResetPosition()
{
	CurrentRow = -1;
	pCurrentRow = NULL;
	CurrentNode = Nil;
}

CVTExtIndex& CVTExtIndex::operator= (CVTExtPosition& Pos)
{
	CurrentRow = Pos.CurrentRow;
	pCurrentRow = Pos.pCurrentRow;
	CurrentNode = Pos.CurrentNode;
	IndexInNode = Pos.IndexInNode;
	SelStarted = Pos.SelStarted;
	return *this;
}

CVTExtIndex& CVTExtIndex::operator= (CVTExtIndex& SrcIndex)
{
	NoRepeat = SrcIndex.NoRepeat;
	IndexFields = SrcIndex.IndexFields;
	Reindex();
	if( SrcIndex.pFilter )
	{
		if( pFilter == NULL )
			pFilter = new CVTExtFilter(KeyRow.GetSize(), &IndexFields);
		*pFilter = *(SrcIndex.pFilter);
	}
	return *this;
}

void CVTExtIndex::Reindex()
{
	RemoveAll();
	if( !IsIndexed() ) return;
	
	CVTIndexRecord idx_rec;
	idx_rec.IndexFields = &IndexFields;

	int num_rows = pVT->GetRowsCount();
	for( int i = 0; i < num_rows; i++ )
	{
		CVTExtRow *row = pVT->GetRow(i);
		idx_rec.Row = row;
		Insert(idx_rec, i);
	}

	ResetPosition();
	SelStarted = false;
}

void CVTExtIndex::Reindex(CString& IndexExpr)
{
	DropFilter();
	IndexFields.Build(IndexExpr, pVT);
	Reindex();
}

void CVTExtIndex::Reindex(CVTExtIndexFields& TheIndexFields)
{
	DropFilter();
	IndexFields = TheIndexFields;
	Reindex();
}

void CVTExtIndex::DropIndex()
{
	DropFilter();
	IndexFields.clear();
	Reindex();
}

bool CVTExtIndex::ColumnIndexed(int col_num)
{
	for( int i = IndexFields.size() - 1; i >= 0; i-- )
	{
		if( IndexFields[i].ColumnNumber == col_num ) return true;
	}
	return false;
}

void CVTExtIndex::RemoveRowFromTree(int row_num, int delta)
{
	if( !IsIndexed() ) return;

	CVTExtIndexTreeNode *node_to_delete = NULL;
	CVTExtIndexTreeNode *node = Min(Root);
	while( node != Nil )
	{
		if( node->ArrayEqualIDs && !node->ArrayEqualIDs->empty() )
		{
			vector<int> &Vector = *(node->ArrayEqualIDs);
			vector<int>::iterator Iterator, End = Vector.end();

			bool vector_erased = false;
			for( Iterator = Vector.begin(); Iterator != End; Iterator++ )
			{
				if( *Iterator == row_num )
				{
					Vector.erase(Iterator); //     .   .
					vector_erased = true;
					node->Count--;
				}
				else if( *Iterator > row_num )
				{
					*Iterator += delta;
				}
			}

			if( node->ID == row_num && !Vector.empty())
			{
				//    ,       -  .
				//     .
				node->ID = Vector[Vector.size()-1];
				node->Key.Row = pVT->GetRow(node->ID);
				Vector.pop_back();
			}

			if( vector_erased )
			{
				if( Vector.empty() )
				{
					delete node->ArrayEqualIDs;
					node->ArrayEqualIDs = NULL;
					UniqueCount++;
				}
				if( delta == 0 ) break; //     ,      
			}
		}
		else if( node->ID == row_num )
		{
			node_to_delete = node;
			if( delta == 0 ) break; //     ,      
		}
		else if( node->ID > row_num )
		{
			node->ID += delta;
		}
		node = Successor(node);
	}

	if( node_to_delete ) Delete(node_to_delete);
}

void CVTExtIndex::RemoveRow(int row_num)
{
	RemoveRowFromTree(row_num, -1);
	//  ,     ,   
	if( row_num == CurrentRow && CurrentNode )
	{
		Next();
	}
}

void CVTExtIndex::RemoveAllRows()
{
	RemoveAll();
	ResetPosition();
}

void CVTExtIndex::InsertRow(CVTExtRow* row, int row_num)
{
	CurrentRow = row_num;
	pCurrentRow = row;

	if( IndexFields.size() == 0 ) return;

	CVTIndexRecord idx_rec;
	idx_rec.IndexFields = &IndexFields;
	idx_rec.Row = row;
	CurrentNode = Insert(idx_rec, row_num);
	IndexInNode = 0;
}

void CVTExtIndex::OnChangeRow(int row_num)
{
	if( IsEmpty() ) return;

	RemoveRowFromTree(row_num, 0);
	InsertRow(pVT->GetRow(row_num), row_num);
}

void CVTExtIndex::OnChangeValue(int row_num, int col_num)
{
	if( IsEmpty() ) return;
	if( !ColumnIndexed(col_num) ) return;

	RemoveRowFromTree(row_num, 0);
	InsertRow(pVT->GetRow(row_num), row_num);
}

void CVTExtIndex::NewColumn()
{
	KeyRow.NewColumn();

	if( pFilter )
		pFilter->NewColumn();
}

void CVTExtIndex::RemoveColumn(int col_num)
{
	for( int i = IndexFields.size() - 1; i >= 0; i-- )
		if( IndexFields[i].ColumnNumber > col_num ) IndexFields[i].ColumnNumber--;

	KeyRow.RemoveColumn(col_num);

	if( pFilter )
		pFilter->RemoveColumn(col_num);
}

void FillSearchKey(CPtrArray* ppValues, CVTExtIndexFields *pIndexFields, CVTExtRow& KeyRow, CVTIndexRecord& IdxRec)
{
	if( ppValues->GetSize() < pIndexFields->size())
		RuntimeError("   !    %i,   %i", pIndexFields->size(), ppValues->GetSize());

	IdxRec.IndexFields = pIndexFields;
	IdxRec.Row = &KeyRow;
	for( int i = pIndexFields->size() - 1; i >= 0; i-- )
	{
		int col_num = pIndexFields->operator[](i).ColumnNumber;
		CValue* val = (CValue*)(ppValues->GetAt(i));
		KeyRow.SetValue(col_num, *val);
	}
}

void FillSearchKey(CValue const& What, CVTExtIndexFields *pIndexFields, CVTExtRow& KeyRow, CVTIndexRecord& IdxRec)
{
	CPtrArray* ppVLArray = CValue2VL(What);
	if( ppVLArray == NULL && pIndexFields->size() > 1 )
		RuntimeError("       !");

	if( ppVLArray == NULL && pIndexFields->size() == 1 )
	{
		IdxRec.IndexFields = pIndexFields;
		IdxRec.Row = &KeyRow;
		int col_num = pIndexFields->operator[](0).ColumnNumber;
		KeyRow.SetValue(col_num, What);
	}
	else
	{
		FillSearchKey(ppVLArray, pIndexFields, KeyRow, IdxRec);
	}
}

int CVTExtIndex::PositioningOnNode(CVTExtIndexTreeNode* node, bool FindLastRow, bool ChangePosition)
{
	if( node )
	{
		int FoundRow, FoundIndexInNode;

		if( FindLastRow && node->ArrayEqualIDs )
		{
			FoundIndexInNode = node->ArrayEqualIDs->size() - 1;
			FoundRow = node->ArrayEqualIDs->operator[](FoundIndexInNode);
		}
		else
		{
			FoundRow = node->ID;
		}

		if( ChangePosition )
		{
			CurrentNode = node;
			IndexInNode = FoundIndexInNode;
			CurrentRow = FoundRow;
			pCurrentRow = pVT->GetRow(CurrentRow);
		}

		return FoundRow;
	}
	return -1;
}

int CVTExtIndex::Find(CValue const& What, bool FindLastRow, bool ChangePosition)
{
	if( !IsIndexed() ) RuntimeError("  !");

	CVTIndexRecord IdxRec;
	FillSearchKey(What, &IndexFields, KeyRow, IdxRec);

	CVTExtIndexTreeNode* node = Search(IdxRec);
	return PositioningOnNode(node, FindLastRow, ChangePosition);
}

CVTExtIndexTreeNode* CVTExtIndex::Find(CPtrArray* pKeyValues)
{
	if( !IsIndexed() ) RuntimeError("  !");

	CVTIndexRecord IdxRec;
	FillSearchKey(pKeyValues, &IndexFields, KeyRow, IdxRec);
	return Search(IdxRec);
}

int CVTExtIndex::FindNearestGE(CValue const& What, bool ChangePosition)
{
	if( !IsIndexed() ) RuntimeError("  !");

	CVTIndexRecord IdxRec;
	FillSearchKey(What, &IndexFields, KeyRow, IdxRec);

	CVTExtIndexTreeNode* node = SearchNearestGE(IdxRec);
	return PositioningOnNode(node, false, ChangePosition);
}

int CVTExtIndex::FindNearestLE(CValue const& What, bool ChangePosition)
{
	if( !IsIndexed() ) RuntimeError("  !");

	CVTIndexRecord IdxRec;
	FillSearchKey(What, &IndexFields, KeyRow, IdxRec);

	CVTExtIndexTreeNode* node = SearchNearestLE(IdxRec);
	return PositioningOnNode(node, true, ChangePosition);
}

int CVTExtIndex::KeyCount(CValue const& What)
{
	if( !IsIndexed() ) RuntimeError("  !");

	CVTIndexRecord IdxRec;
	FillSearchKey(What, &IndexFields, KeyRow, IdxRec);
	CVTExtIndexTreeNode *node = Search(IdxRec);
	if( node )
		return node->Count;
	else
		return -1;
}

CVTExtFilter::CVTExtFilter(int ColumnsCount, CVTExtIndexFields *pIndexFields)
:Row_min(ColumnsCount), Row_max(ColumnsCount)
{
	Rec_min.IndexFields = Rec_max.IndexFields = pIndexFields;
	Rec_min.Row = &Row_min;
	Rec_max.Row = &Row_max;
	bOnlyUnique = false;
}

CVTExtFilter::CVTExtFilter(int ColumnsCount, CVTExtIndexFields *pIndexFields, CValue const& Min, CValue const& Max)
:Row_min(ColumnsCount), Row_max(ColumnsCount)
{
	Rec_min.IndexFields = Rec_max.IndexFields = pIndexFields;
	Set(Min, Max);
	bOnlyUnique = false;
}

void CVTExtFilter::Set(CValue const& Min, CValue const& Max)
{
	FillSearchKey(Min, Rec_min.IndexFields, Row_min, Rec_min);
	FillSearchKey(Max, Rec_max.IndexFields, Row_max, Rec_max);
}

CVTExtFilter& CVTExtFilter::operator= (CVTExtFilter& SrcFilter)
{
	Row_min = SrcFilter.Row_min;
	Row_max = SrcFilter.Row_max;
	bOnlyUnique = SrcFilter.bOnlyUnique;
	return *this;
}

void CVTExtIndex::SetFilter(CValue const& Min, CValue const& Max, bool bUnique)
{
	if( !IsIndexed() )
		RuntimeError("      !");

	if( pFilter )
		pFilter->Set(Min, Max);
	else
		pFilter = new CVTExtFilter(pVT->GetColumnsCount(), &IndexFields, Min, Max);

	pFilter->bOnlyUnique = bUnique;
}

void CVTExtIndex::SetFilter(CValue const& SubSet, int nCols)
{
	if( !IsIndexed() )
		RuntimeError("      !");

	CPtrArray* ppVLArray = CValue2VL(SubSet);
	if( ppVLArray == NULL )
		RuntimeError("       !");

	int nMaxIndex;
	if( nCols >= 0 )
		nMaxIndex = nCols - 1;
	else
		nMaxIndex = min(ppVLArray->GetUpperBound(), IndexFields.size() - 1);
	if( nMaxIndex > ppVLArray->GetUpperBound() )
		RuntimeError("   !");
	if( nMaxIndex >= IndexFields.size() )
		RuntimeError("   ,   !");

	if( pFilter == NULL )
		pFilter = new CVTExtFilter(pVT->GetColumnsCount(), &IndexFields);

	//   ,       .
	int sz = IndexFields.size();
	for( int i = 0; i < sz; i++ )
	{
		int nCol = IndexFields[i].ColumnNumber;
		if( IndexFields[i].Direction >= 0 )
		{
			pFilter->Row_min.GetValue(i).type = 0;
			pFilter->Row_max.GetValue(i).type = USHRT_MAX;
		}
		else
		{
			pFilter->Row_min.GetValue(i).type = USHRT_MAX;
			pFilter->Row_max.GetValue(i).type = 0;
		}
	}

	//     
	for( i = 0; i <= nMaxIndex; i++ )
	{
		int col_num = IndexFields[i].ColumnNumber;
		CValue* val = (CValue*)(ppVLArray->GetAt(i));
		pFilter->Row_min.SetValue(col_num, *val);
		pFilter->Row_max.SetValue(col_num, *val);
	}
}

void CVTExtIndex::DropFilter()
{
	if( pFilter )
	{
		delete pFilter;
		pFilter = NULL;
	}
}

int CVTExtIndex::GetRowsCount()
{
	if( !pFilter ) return Count;

	CVTExtIndexTreeNode *node = SearchNearestGE(pFilter->Rec_min);

	CVTIndexRecord rec;
	rec.IndexFields = &IndexFields;

	bool bUnique = pFilter->bOnlyUnique;
	int nRows = 0;
	while( node != Nil && node != NULL )
	{
		rec.Row = pVT->GetRow(node->ID);
		if( rec.compare(pFilter->Rec_max) > 0 )
			node = NULL;
		else
		{
			if( bUnique )
				nRows++;
			else
				nRows += node->Count;
			node = Successor(node);
		}
	}

	return nRows;
}

bool CVTExtIndex::First()
{
	ResetPosition();
	SelStarted = false;

	if( IsIndexed() )
	{
		if( pFilter == NULL )
			CurrentNode = Min(Root);
		else
			CurrentNode = SearchNearestGE(pFilter->Rec_min);

		if( CurrentNode == NULL ) return false;
	}
	else
	{
		if( pVT->GetRowsCount() <= 0 ) return false;
	}

	SelStarted = true;
	return true;
}

bool CVTExtIndex::Next(bool bUnique)
{
	if( IsIndexed() )
	{
		if( CurrentNode == Nil )
		{
			CurrentRow = -1;
			SelStarted = false;
		}
		else if( SelStarted )
		{
			SelStarted = false;
			CurrentRow = CurrentNode->ID;
			IndexInNode = 0;
		}
		else
		{
			if( pFilter && pFilter->bOnlyUnique ) bUnique = true;
			if( !bUnique && CurrentNode->ArrayEqualIDs && IndexInNode < CurrentNode->ArrayEqualIDs->size() )
			{
				CurrentRow = CurrentNode->ArrayEqualIDs->operator[](IndexInNode++);
			}
			else
			{
				CurrentNode = Successor(CurrentNode);
				if( CurrentNode == Nil )
				{
					CurrentRow = -1;
				}
				else
				{
					CurrentRow = CurrentNode->ID;
					IndexInNode = 0;
				}
			}
		}
	}
	else
	{
		//   
		if( SelStarted )
		{
			SelStarted = false;
			CurrentRow = 0;
		}
		else
		{
			CurrentRow++;
		}

		if( CurrentRow > pVT->Rows.GetUpperBound() )
			CurrentRow = -1;
	}

	if( CurrentRow < 0 )
	{
		ResetPosition();
		return false;
	}

	pCurrentRow = pVT->GetRow(CurrentRow);

	if( pFilter != NULL )
	{
		CVTIndexRecord rec;
		rec.Row = pCurrentRow;
		rec.IndexFields = &IndexFields;
		if( rec.compare(pFilter->Rec_max) > 0 )
		{
			ResetPosition();
			return false;
		}
	}

	return true;
}

bool CVTExtIndex::Last()
{
	ResetPosition();
	SelStarted = false;

	if( IsIndexed() )
	{
		if( pFilter == NULL )
			CurrentNode = Max(Root);
		else
			CurrentNode = SearchNearestLE(pFilter->Rec_max);
		if( CurrentNode == NULL ) return false;
	}
	else
	{
		if( pVT->Rows.GetUpperBound() < 0 ) return false;
	}

	SelStarted = true;
	return true;
}

void CVTExtIndex::GoToNode_Backward(bool bUnique)
{
	if( CurrentNode->ArrayEqualIDs && !bUnique )
	{
		IndexInNode = CurrentNode->ArrayEqualIDs->size() - 1;
		CurrentRow = CurrentNode->ArrayEqualIDs->operator[](IndexInNode);
	}
	else
	{
		CurrentRow = CurrentNode->ID;
		IndexInNode = -1;
	}
}

bool CVTExtIndex::Prev(bool bUnique)
{
	if( IsIndexed() )
	{
		if( CurrentNode == Nil )
		{
			SelStarted = false;
			CurrentRow = -1;
		}
		else if( SelStarted )
		{
			SelStarted = false;
			GoToNode_Backward(bUnique);
		}
		else
		{
			if( pFilter && pFilter->bOnlyUnique ) bUnique = true;
			if( CurrentNode->ArrayEqualIDs && !bUnique && IndexInNode >= 0 )
			{
				if( IndexInNode > 0 )
				{
					CurrentRow = CurrentNode->ArrayEqualIDs->operator[](--IndexInNode);
				}
				else
				{
					IndexInNode--;
					CurrentRow = CurrentNode->ID;
				}
			}
			else
			{
				CurrentNode = Predecessor(CurrentNode);
				if( CurrentNode == Nil )
					CurrentRow = -1;
				else
					GoToNode_Backward(bUnique);
			}
		}
	}
	else
	{
		//   
		if( SelStarted )
		{
			SelStarted = false;
			CurrentRow = pVT->Rows.GetUpperBound();
		}
		else
		{
			CurrentRow--;
		}
	}
	

	if( CurrentRow < 0 )
	{
		ResetPosition();
		return false;
	}

	pCurrentRow = pVT->GetRow(CurrentRow);

	if( pFilter != NULL )
	{
		CVTIndexRecord rec;
		rec.Row = pCurrentRow;
		rec.IndexFields = &IndexFields;
		if( rec.compare(pFilter->Rec_min) < 0 )
		{
			ResetPosition();
			return false;
		}
	}

	return true;
}

void CVTExtIndex::SetCurrentRow(int row_num)
{
	if( SelStarted )
		return; // ,   ,  

	if( row_num < 0 )
	{
		ResetPosition();
		return;
	}

	CurrentRow = row_num;
	pCurrentRow = pVT->GetRow(row_num);

	if( IsIndexed() )
	{
		CVTExtIndexTreeNode *node = Min(Root);
		CVTExtIndexTreeNode *found_node = NULL;
		while( node != Nil )
		{
			if( node->ID == CurrentRow )
			{
				found_node = node;
				IndexInNode = 0;
				break;
			}
			else if( node->ArrayEqualIDs && !node->ArrayEqualIDs->empty() )
			{
				vector<int> &Vector = *(node->ArrayEqualIDs);
				for( int i = Vector.size(); i >= 0; i-- )
				{
					if( Vector[i] == CurrentRow )
					{
						found_node = node;
						IndexInNode = i;
						break;
					}
				}
				if( found_node ) break;
			}
			node = Successor(node);
		}
		CurrentNode = found_node;
	}
}

void CVTExtIndex::SetCurrentValue(int col_num, CValue const& NewVal)
{
	if( pCurrentRow != NULL )
	{
		pCurrentRow->SetValue(col_num, NewVal);
	}
}

void CVTExtIndex::GetCurrentValue(int col_num, CValue& RetVal)
{
	if( pCurrentRow != NULL )
		RetVal = pCurrentRow->GetValue(col_num);
}

void CVTExtIndex::GroupBy(CString Columns)
{
	CVTExtIndexFields SumFields;
	if( !SumFields.Build(Columns, pVT) ) return;

	int nColumns = SumFields.size(), i;
	CVTExtRow *pRow, *pSumRow;
	bool *DeletedRows = new bool[pVT->GetRowsCount()];
	memset((void*)DeletedRows, 0, sizeof(bool) * pVT->GetRowsCount());

	CNumeric** pSums = new CNumeric*[nColumns];

	CVTExtIndexTreeNode *node = Min(Root);
	while( node != Nil )
	{
		if( node->ArrayEqualIDs )
		{
			pSumRow = pVT->GetRow(node->ID);
			for( i = 0; i < nColumns; i++ )
			{
				CValue& val = pSumRow->GetValue(SumFields[i].ColumnNumber);
				if( val.GetTypeCode() != NUMBER_TYPE_1C )
					val = 0L;
				pSums[i] = &(val.m_Number);
			}

			for( int nRow = node->ArrayEqualIDs->size() - 1; nRow >= 0; nRow-- )
			{
				int nRealRowNum = node->ArrayEqualIDs->operator[](nRow);
				pRow = pVT->GetRow(nRealRowNum);
				for( i = 0; i < nColumns; i++ )
				{
					int nCol = SumFields[i].ColumnNumber;
					CNumeric& val = *(pSums[i]);
					CValue& add = pRow->GetValue(nCol);
					val = val + add.m_Number;
				}
				DeletedRows[nRealRowNum] = true;
			}

			delete node->ArrayEqualIDs;
			node->ArrayEqualIDs = NULL;
		}

		node = Successor(node);
	}

	for( i = pVT->GetRowsCount() - 1; i >= 0; i-- )
	{
		if( DeletedRows[i] )
		{
			delete pVT->GetRow(i);
			pVT->Rows.RemoveAt(i);
		}
	}

	delete[] DeletedRows;
	delete[] pSums;
}

void CVTExtIndex::Sum(int col_num, CValue& RetVal)
{
	if( !IsIndexed() || pFilter == NULL )
	{
		RetVal = 0L;
		CNumeric& sum = (CNumeric&)(RetVal.GetNumeric());
		sum = 0L;
		for( int i = pVT->GetRowsCount() - 1; i >= 0; i-- )
			sum = sum + pVT->GetRow(i)->GetValue(col_num).GetNumeric();
		return;
	}

	//Sum by filter
	SumByFilter(col_num, *pFilter, RetVal);
}

void CVTExtIndex::NodeSum(int nCol, CValue& RetVal)
{
	RetVal = 0L;
	CNumeric& sum = (CNumeric&)(RetVal.GetNumeric());
	sum = 0L;
	if( CurrentNode != NULL )
	{
		sum = sum + pVT->GetRow(CurrentNode->ID)->GetValue(nCol).GetNumeric();
		if( CurrentNode->ArrayEqualIDs )
		{
			for( int i = CurrentNode->ArrayEqualIDs->size() - 1; i >= 0; i-- )
				sum = sum + pVT->GetRow(CurrentNode->ArrayEqualIDs->operator[](i))->GetValue(nCol).GetNumeric();
		}
	}
}

void CVTExtIndex::SumByFilter(int col_num, CVTExtFilter& Filter, CValue& RetVal)
{
	RetVal = 0L;
	CNumeric& sum = (CNumeric&)(RetVal.GetNumeric());
	sum = 0L;

	CVTExtIndexTreeNode *node;
	CVTExtIndexTreeNode *last_node = NULL;
	node = SearchNearestGE(Filter.Rec_min);
	last_node = SearchNearestLE(Filter.Rec_max);
	if( node == NULL || last_node == NULL ) return; // -  

	bool bUnique = Filter.bOnlyUnique;
	while( node != Nil )
	{
		sum = sum + pVT->GetRow(node->ID)->GetValue(col_num).GetNumeric();
		if( !bUnique && node->ArrayEqualIDs )
		{
			for( int i = node->ArrayEqualIDs->size() - 1; i >= 0; i-- )
				sum = sum + pVT->GetRow(node->ArrayEqualIDs->operator[](i))->GetValue(col_num).GetNumeric();
		}

		if( node == last_node )
			node = Nil;
		else
			node = Successor(node);
	}
}

void CVTExtIndex::SumByRange(int col_num, CValue& min, CValue& max, CValue& RetVal)
{
	CVTExtFilter Filter(pVT->GetColumnsCount(), &IndexFields, min, max);
	SumByFilter(col_num, Filter, RetVal);
}

///////////////////////////////////////////////////////////////////////////
////  CVTExtIndexCollection  //////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
CVTExtIndex* CVTExtIndexCollection::operator [](int nIndex) const
{
	return (CVTExtIndex*)(CCollection::operator [](nIndex));
}

CVTExtIndex* CVTExtIndexCollection::operator [](LPCSTR szName) const
{
	return (CVTExtIndex*)(CCollection::operator [](szName));
}

CVTExtIndex* CVTExtIndexCollection::operator [](CValue* Index) const
{
	CVTExtIndex* pVTIndex = NULL;
	switch( Index->GetTypeCode() )
	{
	case NUMBER_TYPE_1C:
		{
			int nIndex = (int)(Index->GetNumeric());
			pVTIndex = operator [](nIndex - 1);
			if( pVTIndex == NULL )
				RuntimeError("   %i   !", nIndex);
		}
		break;
	case STRING_TYPE_1C:
		{
			LPCSTR szIndex = Index->GetString();
			pVTIndex = operator [](szIndex);
			if( pVTIndex == NULL )
				RuntimeError("   %s   !", szIndex);
		}
		break;
	default:
		RuntimeError("       !  (%i)", Index->GetTypeCode());
	}

	return pVTIndex;
}


int CVTExtIndexCollection::AddIndex(CVTExtIndex* pIndex)
{
	Add(pIndex);
	return GetCount() - 1;
}

int CVTExtIndexCollection::AddIndex(CString const& IndexName, CVTExtIndexFields const& IndexFields, bool bUnique, CVTExtended* pVT)
{
	if( IndexName.IsEmpty() )
		RuntimeError("   !");
	if( GetItem(IndexName) != NULL )
		RuntimeError(" %s  !", IndexName);

	CVTExtIndex* pIndex = new CVTExtIndex(IndexName, IndexFields, bUnique, pVT);
	return AddIndex(pIndex);
}

int CVTExtIndexCollection::AddIndex(CString const& IndexName, CString& IndexExpr, bool bUnique, CVTExtended* pVT)
{
	if( IndexName.IsEmpty() )
		RuntimeError("   !");
	if( GetItem(IndexName) != NULL )
		RuntimeError(" %s  !", IndexName);

	CVTExtIndex* pIndex = new CVTExtIndex(IndexName, IndexExpr, bUnique, pVT);
	return AddIndex(pIndex);
}


void CVTExtIndexCollection::RemoveIndex(int nIndex)
{
	CVTExtIndex* pIndex = operator[](nIndex);
	Remove(nIndex);
	delete pIndex;
}

void CVTExtIndexCollection::RemoveIndex(CVTExtIndex* pIndex)
{
	int nIndex = GetIndex(pIndex);
	Remove(nIndex);
	delete pIndex;
}

void CVTExtIndexCollection::RemoveAll()
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		RemoveIndex(i);
	}
}

void CVTExtIndexCollection::Reindex()
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->Reindex();
	}
}

bool CVTExtIndexCollection::ColumnIndexed(int col_num)
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		if( operator[](i)->ColumnIndexed(col_num) ) return true;
	}
	return false;
}

void CVTExtIndexCollection::NewColumn()
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->NewColumn();
	}
}

void CVTExtIndexCollection::RemoveColumn(int col_num)
{
	if( ColumnIndexed(col_num) )
		RuntimeError("  ,    !");

	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->RemoveColumn(col_num);
	}
}


void CVTExtIndexCollection::InsertRow(CVTExtRow* row, int row_num)
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->InsertRow(row, row_num);
	}
}

void CVTExtIndexCollection::RemoveRow(int row_num)
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->RemoveRow(row_num);
	}
}

void CVTExtIndexCollection::RemoveAllRows()
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->RemoveAllRows();
	}
}

void CVTExtIndexCollection::OnChangeValue(int row_num, int col_num)
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->OnChangeValue(row_num, col_num);
	}
}

void CVTExtIndexCollection::OnChangeRow(int row_num)
{
	for( int i = GetUpperBound(); i >= 0; i-- )
	{
		operator[](i)->OnChangeRow(row_num);
	}
}
